package esame.dao;

import esame.dao.hsqldb.HsqldbDAOFactory;

public abstract class DAOFactory {

	// --- List of supported DAO types ---
	
	/**
	 * Numeric constant '1' corresponds to explicit Hsqldb choice
	 */
	public static final int HSQLDB = 1;

	
	// --- Actual factory method ---
	
	/**
	 * Depending on the input parameter
	 * this method returns one out of several possible 
	 * implementations of this factory spec.
	 */
	public static DAOFactory getDAOFactory(int whichFactory) {
		switch ( whichFactory ) {
		case HSQLDB:
			return new HsqldbDAOFactory();
		default:
			System.err.println("Database type unknown.");
			return null;
		}
	}
	
	/**
	 * Depending on the input parameter
	 * this method returns one out of several possible 
	 * implementations of this factory spec.
	 * @param whichFactory fully qualified class name of the require instance.
	 * @return DAOFactory
	 */
	public static DAOFactory getDAOFactory(String whichFactory) 
	{
		try {
			return (DAOFactory) Class.forName(whichFactory).newInstance();
		} catch (Exception e) {
			System.err.println(e.getMessage());
			return null;
		} 
	}
	
	
	
	// --- Factory specification: concrete factories implementing this spec must provide this methods! ---

	/**
	 * Method to obtain a DATA ACCESS OBJECT
	 * for the datatype 'Testo'
	 */
	public abstract TestoDAO getTestoDAO();

}
