package esame.dao.hsqldb;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;

import esame.dao.TestoDAO;
import esame.dao.TestoDTO;

public class HsqldbTestoDAO implements TestoDAO {
	
	// === Costanti letterali per non sbagliarsi a scrivere !!! ============================
	
	static final String TABLE = "testi";

	// -------------------------------------------------------------------------------------

	static final String ID = "id";
	static final String TESTO = "testo";
	
	// == STATEMENT SQL ====================================================================

	// INSERT INTO table ( name,description, ...) VALUES ( ?,?, ... );
	static final String INSERT = 
		"INSERT " +
			"INTO " + TABLE + " ( " + 
			TESTO + " " +
			") " +
			"VALUES (?) "
		;
	
	// SELECT * FROM table WHERE idcolumn = ?;
	static String READ_BY_ID = 
		"SELECT * " +
			"FROM " + TABLE + " " +
			"WHERE " + ID + " = ? "
		;

	// SELECT * FROM table;
	static String READ_ALL = 
		"SELECT * " +
		"FROM " + TABLE + " ";

	static String LAST_INSERT = 
		"SELECT MAX(id) FROM "+TABLE+" ;"
		;
	
	// -------------------------------------------------------------------------------------

	// CALL IDENTITY();
	/**
	 * Restituisce l'identificativo dell'ultima tupla inserita. 
	 * TIPOCO ESEMPIO DI FUNZIONE >>>NON<<< PORTABILE: NON CROSS-DATABASE!!!
	 * Questa versione funziona su hibernate
	 */
	static String LAST_INSERT_HSQLDB = 
		"CALL " + 
			"IDENTITY() ;"
		;

	// -------------------------------------------------------------------------------------

	// CREATE entrytable ( idcolumn INT NOT NULL IDENTITY, ..., UNIQUE ( idcolumn ) );
	static String CREATE_TABLE = 
		"CREATE " +
			"TABLE " + TABLE +" ( " +
				ID + " BIGINT NOT NULL IDENTITY, " +
				TESTO + " LONGVARCHAR, " +
				"UNIQUE( " + ID + " ) " +
			") "
		;
	static String DROP_TABLE = 
		"DROP " +
			"TABLE " + TABLE + " "
		;
	
	public HsqldbTestoDAO()
	{
		
	}

	@Override
	public boolean create(TestoDTO testo) {
		boolean result=false;
		
		Connection conn=HsqldbDAOFactory.createConnection();
		if(conn==null)
			return false;
		
		try
		{
			PreparedStatement ps=conn.prepareStatement(HsqldbTestoDAO.INSERT);
			ps.setString(1, testo.getTesto());
			
			result=ps.executeUpdate()>0;
			ps.close();
		}
		catch(Exception e)
		{
			
		}
		finally
		{
			HsqldbDAOFactory.closeConnection(conn);
		}
		return result;
	}

	@Override
	public boolean createTable() 
	{
		boolean result=false;
		
		Connection conn=HsqldbDAOFactory.createConnection();
		if(conn==null)
			return false;
		
		try
		{
			Statement st=conn.createStatement();
			
			result=st.executeUpdate(HsqldbTestoDAO.CREATE_TABLE)>0;
			st.close();
		}
		catch(Exception e)
		{
			
		}
		finally
		{
			HsqldbDAOFactory.closeConnection(conn);
		}
		return result;
	}

	@Override
	public boolean delete(Long id) {
		throw new UnsupportedOperationException("Metodo non implementato.");
	}

	@Override
	public boolean dropTable() 
	{
		boolean result=false;
		
		Connection conn=HsqldbDAOFactory.createConnection();
		if(conn==null)
			return false;
		
		try
		{
			Statement st=conn.createStatement();
			
			result=st.executeUpdate(HsqldbTestoDAO.DROP_TABLE)>0;
			st.close();
		}
		catch(Exception e)
		{
			
		}
		finally
		{
			HsqldbDAOFactory.closeConnection(conn);
		}
		return result;
	}

	@Override
	public TestoDTO read(Long id) {
		throw new UnsupportedOperationException("Metodo non implementato.");
	}

	@Override
	public boolean update(TestoDTO testo) {
		throw new UnsupportedOperationException("Metodo non implementato.");
	}

	@Override
	public Long getLastInsertedId() 
	{
		Long result=-1L;
		
		Connection conn=HsqldbDAOFactory.createConnection();
		if(conn==null)
			return -2L;
		
		try
		{
			Statement st = conn.createStatement();
			ResultSet rs = st.executeQuery(HsqldbTestoDAO.LAST_INSERT);
			if ( rs.next() ) {
				result = rs.getLong(1);
			}		
			rs.close();
			st.close();
		}
		catch(Exception e)
		{
			
		}
		finally
		{
			HsqldbDAOFactory.closeConnection(conn);
		}
		return result;
	}

}
